@@ -43,6 +43,7 @@
 #include "goo-window.h"
 #include "goo-volume-button.h"
 #include "gtk-utils.h"
+#include "gtk-file-chooser-preview.h"
 #include "glib-utils.h"
 #include "gconf-utils.h"
 #include "main.h"
@@ -119,6 +120,8 @@ struct _GooWindowPrivateData {
 	guint            check_id;
 
 	GList           *url_list;
+
+	GtkWidget       *preview;
 };
 
 static int icon_size = 0;
@@ -273,16 +276,13 @@ window_update_sensitivity (GooWindow *window)
 	set_sensitive (window, "Properties", !error && (discid != NULL));
 	set_sensitive (window, "PickCoverFromDisk", !error && (discid != NULL));
 	set_sensitive (window, "RemoveCover", !error && (discid != NULL));
-#ifdef HAVE_NEON
 	set_sensitive (window, "SearchCoverFromWeb", !error && (discid != NULL));
-#else
-	set_sensitive (window, "SearchCoverFromWeb", FALSE);
-#endif /* HAVE_NEON*/
 
-
-	goo_player_info_set_sensitive (GOO_PLAYER_INFO (window->priv->info), !error && (discid != NULL));
+	goo_player_info_set_sensitive (GOO_PLAYER_INFO (priv->info), !error && (discid != NULL));
 	set_sensitive (window, "Repeat", play_all);
 	set_sensitive (window, "Shuffle", play_all);
+
+	gtk_widget_set_sensitive (priv->list_view, discid != NULL);
 }
 
 
@@ -528,9 +528,8 @@ add_columns (GtkTreeView *treeview)
                                              "pixbuf", COLUMN_ICON,
                                              NULL);
 
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
-	gtk_tree_view_column_set_fixed_width (column, 40); /* FIXME */
 
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_NUMBER);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
@@ -546,7 +545,6 @@ add_columns (GtkTreeView *treeview)
 
 		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 		gtk_tree_view_column_set_resizable (column, TRUE);
-		gtk_tree_view_column_set_min_width (column, 50); /* FIXME */
 
 		gtk_tree_view_column_set_sort_column_id (column, i);
 
@@ -651,6 +649,111 @@ pref_view_playlist_changed (GConfClient *client,
 }
 
 
+static void
+print_playlist (GooWindow *window)
+{
+	GList *scan;
+
+	debug (DEBUG_INFO, "PLAYLIST: ");
+	for (scan = window->priv->playlist; scan; scan = scan->next) 
+		debug (DEBUG_INFO, "%d, ", GPOINTER_TO_INT (scan->data));
+	debug (DEBUG_INFO, "\n");
+}
+
+
+static void
+create_playlist (GooWindow *window,
+		 gboolean   play_all,
+		 gboolean   shuffle)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GList *playlist;
+	int    pos = 0, i, n;
+
+	debug (DEBUG_INFO, "PLAY ALL: %d\n", play_all);
+	debug (DEBUG_INFO, "SHUFFLE: %d\n", shuffle);
+
+	if (priv->playlist != NULL)
+		g_list_free (priv->playlist);
+	priv->playlist = NULL;
+
+	if (!play_all) 
+		return;
+
+	playlist = NULL;
+
+	if (priv->current_song != NULL)
+		pos = priv->current_song->number;
+	n = g_list_length (priv->song_list);
+
+	for (i = 0; i < n; i++, pos = (pos + 1) % n) {
+		if ((priv->current_song != NULL) 
+		    && (priv->current_song->number == pos))
+			continue;
+		playlist = g_list_prepend (playlist, GINT_TO_POINTER (pos));
+	}
+
+	playlist = g_list_reverse (playlist);
+
+	if (shuffle) {
+		GRand *grand = g_rand_new ();
+		GList *random_list = NULL;
+		int    len = g_list_length (playlist);
+
+		while (playlist != NULL) {
+			GList *item;
+
+			pos = g_rand_int_range (grand, 0, len--);
+			item = g_list_nth (playlist, pos);
+			playlist = g_list_remove_link (playlist, item);
+			random_list = g_list_concat (random_list, item);
+		}
+		g_rand_free (grand);
+
+		playlist = random_list;
+	} 
+
+	priv->playlist = playlist;
+}
+
+
+static void
+play_next_song_in_playlist (GooWindow *window)
+{
+	GooWindowPrivateData *priv = window->priv;
+	gboolean  play_all;
+	gboolean  shuffle;
+	gboolean  repeat;
+	GList    *next = NULL;
+
+	play_all = eel_gconf_get_boolean (PREF_PLAYLIST_PLAYALL, TRUE);
+	shuffle  = eel_gconf_get_boolean (PREF_PLAYLIST_SHUFFLE, FALSE);
+	repeat = eel_gconf_get_boolean (PREF_PLAYLIST_REPEAT, FALSE);
+
+	next = priv->playlist;
+
+	if ((next == NULL) && repeat) {
+		if (priv->current_song != NULL) {
+			song_info_unref (priv->current_song);
+			priv->current_song = NULL;
+		}
+		create_playlist (window, play_all, shuffle);
+		next = priv->playlist;
+	}
+	
+	print_playlist (window);
+
+	if (next == NULL)
+		goo_player_stop (priv->player);
+	else {
+		int pos = GPOINTER_TO_INT (next->data);
+		goo_player_seek_song (priv->player, pos);
+		priv->playlist = g_list_remove_link (priv->playlist, next);
+		g_list_free (next);
+	}
+}
+
+
 static void
 pref_playlist_playall_changed (GConfClient *client,
 			       guint        cnxn_id,
@@ -658,11 +761,18 @@ pref_playlist_playall_changed (GConfClient *client,
 			       gpointer     user_data)
 {
 	GooWindow *window = user_data;
+	gboolean   play_all;
+	gboolean   shuffle;
 
 	g_return_if_fail (window != NULL);
 
-	set_active (window, "PlayAll", eel_gconf_get_boolean (PREF_PLAYLIST_PLAYALL, TRUE));
+	play_all = eel_gconf_get_boolean (PREF_PLAYLIST_PLAYALL, TRUE);
+	shuffle = eel_gconf_get_boolean (PREF_PLAYLIST_SHUFFLE, FALSE);
+	create_playlist (window, play_all, shuffle);
+
 	window_update_sensitivity (window);
+
+	set_active (window, "PlayAll", play_all);
 }
 
 
@@ -687,10 +797,16 @@ pref_playlist_shuffle_changed (GConfClient *client,
 			       gpointer     user_data)
 {
 	GooWindow *window = user_data;
+	gboolean   play_all;
+	gboolean   shuffle;
 
 	g_return_if_fail (window != NULL);
 
-	set_active (window, "Shuffle", eel_gconf_get_boolean (PREF_PLAYLIST_SHUFFLE, FALSE));
+	play_all = eel_gconf_get_boolean (PREF_PLAYLIST_PLAYALL, TRUE);
+	shuffle = eel_gconf_get_boolean (PREF_PLAYLIST_SHUFFLE, FALSE);
+	create_playlist (window, play_all, shuffle);
+
+	set_active (window, "Shuffle", shuffle);
 }
 
 
@@ -1029,111 +1145,6 @@ goo_window_select_current_song (GooWindow *window)
 }
 
 
-static void
-print_playlist (GooWindow *window)
-{
-	GList *scan;
-
-	debug (DEBUG_INFO, "PLAYLIST: ");
-	for (scan = window->priv->playlist; scan; scan = scan->next) 
-		debug (DEBUG_INFO, "%d, ", GPOINTER_TO_INT (scan->data));
-	debug (DEBUG_INFO, "\n");
-}
-
-
-static void
-create_playlist (GooWindow *window,
-		 gboolean   play_all,
-		 gboolean   shuffle)
-{
-	GooWindowPrivateData *priv = window->priv;
-	GList *playlist;
-	int    pos = 0, i, n;
-
-	debug (DEBUG_INFO, "PLAY ALL: %d\n", play_all);
-	debug (DEBUG_INFO, "SHUFFLE: %d\n", shuffle);
-
-	if (priv->playlist != NULL)
-		g_list_free (priv->playlist);
-	priv->playlist = NULL;
-
-	if (!play_all) 
-		return;
-
-	playlist = NULL;
-
-	if (priv->current_song != NULL)
-		pos = priv->current_song->number;
-	n = g_list_length (priv->song_list);
-
-	for (i = 0; i < n; i++, pos = (pos + 1) % n) {
-		if ((priv->current_song != NULL) 
-		    && (priv->current_song->number == pos))
-			continue;
-		playlist = g_list_prepend (playlist, GINT_TO_POINTER (pos));
-	}
-
-	playlist = g_list_reverse (playlist);
-
-	if (shuffle) {
-		GRand *grand = g_rand_new ();
-		GList *random_list = NULL;
-		int    len = g_list_length (playlist);
-
-		while (playlist != NULL) {
-			GList *item;
-
-			pos = g_rand_int_range (grand, 0, len--);
-			item = g_list_nth (playlist, pos);
-			playlist = g_list_remove_link (playlist, item);
-			random_list = g_list_concat (random_list, item);
-		}
-		g_rand_free (grand);
-
-		playlist = random_list;
-	} 
-
-	priv->playlist = playlist;
-}
-
-
-static void
-play_next_song_in_playlist (GooWindow *window)
-{
-	GooWindowPrivateData *priv = window->priv;
-	gboolean  play_all;
-	gboolean  shuffle;
-	gboolean  repeat;
-	GList    *next = NULL;
-
-	play_all = eel_gconf_get_boolean (PREF_PLAYLIST_PLAYALL, TRUE);
-	shuffle  = eel_gconf_get_boolean (PREF_PLAYLIST_SHUFFLE, FALSE);
-	repeat = eel_gconf_get_boolean (PREF_PLAYLIST_REPEAT, FALSE);
-
-	next = priv->playlist;
-
-	if ((next == NULL) && repeat) {
-		if (priv->current_song != NULL) {
-			song_info_unref (priv->current_song);
-			priv->current_song = NULL;
-		}
-		create_playlist (window, play_all, shuffle);
-		next = priv->playlist;
-	}
-	
-	print_playlist (window);
-
-	if (next == NULL)
-		goo_player_stop (priv->player);
-	else {
-		int pos = GPOINTER_TO_INT (next->data);
-		goo_player_seek_song (priv->player, pos);
-		priv->playlist = g_list_remove_link (priv->playlist, next);
-		g_list_free (next);
-	}
-}
-
-
 static gboolean
 next_time_idle (gpointer callback_data)
 {
@@ -1226,8 +1237,8 @@ window_update_title (GooWindow *window)
 }
 
 
-static char *
-get_disc_cover_filename (GooWindow *window)
+char *
+goo_window_get_cover_filename (GooWindow *window)
 {
 	GooPlayerCD *player_cd = GOO_PLAYER_CD (window->priv->player);
 	char        *discid;
@@ -1250,10 +1261,12 @@ goo_window_update_cover (GooWindow *window)
 	char      *filename;
 	GdkPixbuf *image;
 	
-	filename = get_disc_cover_filename (window);
-	if (filename == NULL)
+	filename = goo_window_get_cover_filename (window);
+	if (filename == NULL) {
 		goo_player_info_set_cover (GOO_PLAYER_INFO (window->priv->info), NULL);
-	
+		return;
+	}
+
 	image = gdk_pixbuf_new_from_file (filename, NULL);
 	if (image != NULL) {
 		goo_player_info_set_cover (GOO_PLAYER_INFO (window->priv->info), image);
@@ -2465,7 +2478,7 @@ goo_window_set_cover_image (GooWindow  *window,
 				      frame,
 				      1, 1);
 
-		cover_filename = get_disc_cover_filename (window);
+		cover_filename = goo_window_get_cover_filename (window);
 		debug (DEBUG_INFO, "SAVE IMAGE %s\n", cover_filename);
 		
 		if (! gdk_pixbuf_save (frame, cover_filename, "png", &error, NULL))
@@ -2485,6 +2498,20 @@ goo_window_set_cover_image (GooWindow  *window,
 }
 
 
+static void
+open_update_preview_cb (GtkFileChooser *file_sel,
+			gpointer        user_data)
+{
+	GooWindow *window = user_data;
+	char      *uri;
+
+	uri = gtk_file_chooser_get_preview_uri (file_sel);
+	debug (DEBUG_INFO, "PREVIEW: %s", uri);
+	gtk_file_chooser_preview_set_uri (GTK_FILE_CHOOSER_PREVIEW (window->priv->preview), uri);
+	g_free (uri);
+}
+
+
 static void
 open_response_cb (GtkDialog  *file_sel,
 		  int         button_number,
@@ -2531,6 +2558,11 @@ goo_window_pick_cover_from_disk (GooWindow *window)
 	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), TRUE);
 
+	window->priv->preview = gtk_file_chooser_preview_new ();
+	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (file_sel), window->priv->preview);
+	gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (file_sel), FALSE);
+	gtk_file_chooser_set_preview_widget_active (GTK_FILE_CHOOSER (file_sel), TRUE);
+
 	path = eel_gconf_get_path (PREF_GENERAL_COVER_PATH, "~");
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel), path);
 	g_free (path);
@@ -2552,6 +2584,10 @@ goo_window_pick_cover_from_disk (GooWindow *window)
 			  "response",
 			  G_CALLBACK (open_response_cb),
 			  window);
+	g_signal_connect (G_OBJECT (file_sel),
+			  "update-preview",
+			  G_CALLBACK (open_update_preview_cb),
+			  window);
 	g_signal_connect_swapped (GTK_DIALOG (file_sel),
 				  "close",
 				  G_CALLBACK (gtk_widget_destroy),
@@ -2564,7 +2600,6 @@ goo_window_pick_cover_from_disk (GooWindow *window)
 void
 goo_window_search_cover_on_internet (GooWindow *window)
 {
-#ifdef HAVE_NEON
 	const char *album, *artist;
 
 	debug (DEBUG_INFO, "SEARCH ON INTERNET\n");
@@ -2576,8 +2611,6 @@ goo_window_search_cover_on_internet (GooWindow *window)
 		return; /*FIXME*/
 
 	dlg_cover_chooser (window, album, artist);
-
-#endif /* HAVE_NEON*/
 }
 
 
@@ -2586,7 +2619,7 @@ goo_window_remove_cover (GooWindow   *window)
 {
 	char *cover_filename;
 		
-	cover_filename = get_disc_cover_filename (window);
+	cover_filename = goo_window_get_cover_filename (window);
 	if (cover_filename == NULL)
 		return;
 
