@@ -38,9 +38,7 @@
 #include "goo-application.h"
 #include "gtk-utils.h"
 
-#ifdef HAVE_NEON
-#include <ne_socket.h>
-#endif /* HAVE_NEON*/
+GtkWindow *main_window = NULL;
 
 static void     prepare_app         (poptContext pctx);
 static void     initialize_data     (void);
@@ -49,13 +47,10 @@ static void     release_data        (void);
 static void     init_session        (char **argv);
 static gboolean session_is_restored (void);
 static gboolean load_session        (void);
+
+static char         *default_device = NULL;
 static BonoboObject *goo_application = NULL;
 
-GtkWindow *main_window = NULL;
-
-static char *default_device = NULL;
-
-
 struct poptOption options[] = {
 	{ "device", 'd', POPT_ARG_STRING, &default_device, 0,
 	  N_("CD device to be used"),
@@ -127,10 +122,6 @@ int main (int argc, char **argv)
 	gst_init (NULL, NULL);
 	/*gst_use_threads (TRUE);*/
 
-#ifdef HAVE_NEON
-	ne_sock_init ();
-#endif /* HAVE_NEON*/
-
 	goo_stock_init ();
 	init_session (argv);
 	initialize_data ();
